/*
 * Copyright 2010 Gal Dolber.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.guit.client;

import com.google.inject.Inject;

import com.guit.client.binder.ViewBinder;
import com.guit.client.command.CommandService;
import com.guit.client.eventbus.EventBus;
import com.guit.client.place.PlaceManager;

/**
 * Abstract presenter with EventBus, CommandService and PlaceManager support.
 * @param <P> Presenter class.
 * @param <V> View class.
 * @param <B> Binder class.
 */
@Deprecated
public abstract class AbstractPresenter<P extends Presenter, V extends View, B extends ViewBinder<P, V>> implements Presenter {

    protected EventBus eventBus;
    protected CommandService commandService;
    protected PlaceManager placeManager;
    protected V view;
    private B binder;

    /**
     * Bind all the events. Should not call this unless you called unbind().
     */
    @SuppressWarnings("unchecked")
    protected void bind() {
        binder.bind((P) this, view, eventBus);
    }

    @Override
    public View getView() {
        return view;
    }

    @Inject
    public void inject(V view, EventBus eventBus, CommandService commandService, PlaceManager placeManager, B binder) {
        this.view = view;
        this.eventBus = eventBus;
        this.commandService = commandService;
        this.placeManager = placeManager;
        this.binder = binder;
        bind();
    }

    /**
     * Unbind all the events.
     */
    protected void unbind() {
        binder.unbind();
    }
    
    @Override
    public void releaseView() {
        assert false: "releaseView is only available in GuitPresenter";
    }
    
    @Override
    public void destroy() {
        assert false: "destroy is only available in GuitPresenter";
    }
}
