/*
 * Copyright 2010 Gal Dolber.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.guit.client.async.event;

import com.google.gwt.event.shared.GwtEvent;

public class AsyncActivityEvent extends GwtEvent<AsyncActivityHandler> {

    private static Type<AsyncActivityHandler> type;

    public static final AsyncActivityEvent START = new AsyncActivityEvent(AsyncActivityType.START);
    public static final AsyncActivityEvent END = new AsyncActivityEvent(AsyncActivityType.END);

    public static Type<AsyncActivityHandler> getType() {
        return type != null ? type : (type = new Type<AsyncActivityHandler>());
    }

    private final AsyncActivityType activityType;

    private AsyncActivityEvent(AsyncActivityType activityType) {
        this.activityType = activityType;
    }

    @Override
    protected void dispatch(AsyncActivityHandler handler) {
        handler.onRpcEvent(this);
    }

    public AsyncActivityType getActivityType() {
        return activityType;
    }

    @Override
    public Type<AsyncActivityHandler> getAssociatedType() {
        return getType();
    }

    @Override
    public String toDebugString() {
        return super.toDebugString() + activityType.toString();
    }
}
