/*
 * Copyright 2010 Gal Dolber.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.guit.client.command;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.rpc.SerializationException;

import com.guit.client.async.event.AsyncExceptionEvent;
import com.guit.client.command.action.Action;
import com.guit.client.command.action.BatchResponse;
import com.guit.client.command.action.CommandException;
import com.guit.client.command.action.CommandSerializable;
import com.guit.client.command.action.Response;
import com.guit.client.eventbus.EventBus;

import java.util.ArrayList;
import java.util.List;

public class CommandBatchRequestUnit extends AbstractCommandRequestUnit implements AsyncCallback<BatchResponse> {

    private final CommandRequestUnitFinish<CommandBatchRequestUnit> finishCommand;

    private List<AsyncCallback<Response>> callback;

    private ArrayList<Action<Response>> actions;

    public CommandBatchRequestUnit(EventBus eventBus, CommandRpcAsync service,
        CommandRequestUnitFinish<CommandBatchRequestUnit> finishCommand) {
        super(eventBus, service);
        this.finishCommand = finishCommand;
    }

    public void execute(ArrayList<Action<Response>> a, List<AsyncCallback<Response>> c) {
        this.actions = a;
        this.callback = c;
        service.executeBatch(a, this);
    }

    @Override
    public void onFailure(Throwable caught) {
        if (!GWT.isScript()) {
            if (caught instanceof SerializationException) {
                StringBuilder sb = new StringBuilder();
                for (Action<Response> a : actions) {
                    if (sb.length() > 0) {
                        sb.append(" or ");
                    }
                    sb.append(a.getClass().getName());
                }
                throw new RuntimeException("Some of your actions or responses contains non-serializable data. Found: " + sb.toString());
            }
        }
        
        eventBus.fireEvent(new AsyncExceptionEvent(caught));
        finishCommand.onFinish(CommandBatchRequestUnit.this);
    }

    @Override
    public void onSuccess(BatchResponse result) {
        ArrayList<CommandSerializable> responses = result.getResponses();
        for (int n = 0; n < responses.size(); n++) {
            CommandSerializable response = responses.get(n);
            AsyncCallback<Response> asyncCallback = callback.get(n);
            if (response instanceof Response) {
                asyncCallback.onSuccess((Response) response);
            } else {
                eventBus.fireEvent(new AsyncExceptionEvent((CommandException) response));
                asyncCallback.onFailure((Throwable) response);
            }
        }
        finishCommand.onFinish(CommandBatchRequestUnit.this);
    }
}
