/*
 * Copyright 2010 Gal Dolber.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.guit.client.command;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.rpc.SerializationException;

import com.guit.client.async.event.AsyncExceptionEvent;
import com.guit.client.command.action.Action;
import com.guit.client.command.action.Response;
import com.guit.client.eventbus.EventBus;

public class CommandSingleRequestUnit extends AbstractCommandRequestUnit implements AsyncCallback<Response> {

    private final CommandRequestUnitFinish<CommandSingleRequestUnit> finishCommand;

    private AsyncCallback<Response> callback;
    
    // Used for SerializationException trace
    private Class<?> action;

    public CommandSingleRequestUnit(EventBus eventBus, CommandRpcAsync service,
        CommandRequestUnitFinish<CommandSingleRequestUnit> finishCommand) {
        super(eventBus, service);
        this.finishCommand = finishCommand;
    }

    public void execute(Action<Response> a, AsyncCallback<Response> c) {
        this.action = a.getClass();
        this.callback = c;
        service.execute(a, this);
    }

    @Override
    public void onFailure(Throwable caught) {
        if (!GWT.isScript()) {
            if (caught instanceof SerializationException) {
                throw new RuntimeException("Some of your actions or responses contains non-serializable data. Found: " + action.getName());
            }
        }
        
        eventBus.fireEvent(new AsyncExceptionEvent(caught));
        callback.onFailure(caught);
        finishCommand.onFinish(CommandSingleRequestUnit.this);
    }

    @Override
    public void onSuccess(Response result) {
        callback.onSuccess(result);
        finishCommand.onFinish(CommandSingleRequestUnit.this);
    }
}
