/*
 * Copyright 2010 Gal Dolber.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.guit.client.eventbus;

import com.google.gwt.event.shared.EventHandler;
import com.google.gwt.event.shared.GwtEvent;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.event.shared.SimpleEventBus;

public class EventBusImpl implements EventBus {

    private final SimpleEventBus handlerManager = new SimpleEventBus();

    /**
     * Adds a handler to an event.
     * @param <H> Handler type
     * @param handler Handler
     * @param type Type
     * @return Handler registration
     */
    public <H extends EventHandler> HandlerRegistration addHandler(final H handler, GwtEvent.Type<H> type) {
        return handlerManager.addHandler(type, handler);
    }

    /**
     * Fires an event.
     * @param event Event
     */
    public void fireEvent(GwtEvent<?> event) {
        handlerManager.fireEvent(event);
    }
}
