/*
 * Copyright 2010 Gal Dolber.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.guit.client.place.event;

import com.google.gwt.event.shared.GwtEvent;

import com.guit.client.place.Place;

public class PlaceGoEvent extends GwtEvent<PlaceGoHandler> {

    private static Type<PlaceGoHandler> type;

    public static Type<PlaceGoHandler> getType() {
        return type != null ? type : (type = new Type<PlaceGoHandler>());
    }

    private final Class<? extends Place<?>> placeClass;

    private final Object placeData;

    private final String token;

    public <D> PlaceGoEvent(Class<? extends Place<D>> placeClass, D placeData) {
        this.placeClass = placeClass;
        this.placeData = placeData;
        this.token = null;
    }

    public PlaceGoEvent(String token) {
        this.token = token;
        this.placeClass = null;
        this.placeData = null;
    }

    @Override
    protected void dispatch(PlaceGoHandler handler) {
        handler.onPlaceGo(this);
    }

    @Override
    public Type<PlaceGoHandler> getAssociatedType() {
        return getType();
    }

    public Class<? extends Place<?>> getPlaceClass() {
        return placeClass;
    }

    public Object getPlaceData() {
        return placeData;
    }

    public String getToken() {
        return token;
    }
}
