/*
 * Copyright 2010 Gal Dolber.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.guit.client.place.event;

import com.google.gwt.event.shared.GwtEvent;

import com.guit.client.place.Place;

public class PlaceNewItemEvent extends GwtEvent<PlaceNewItemHandler> {

    private static Type<PlaceNewItemHandler> type;

    public static Type<PlaceNewItemHandler> getType() {
        return type != null ? type : (type = new Type<PlaceNewItemHandler>());
    }

    private final Class<? extends Place<?>> placeClass;

    private final Object placeData;

    private final String token;

    public <D> PlaceNewItemEvent(Class<? extends Place<D>> placeClass, D placeData) {
        this.token = null;
        this.placeClass = placeClass;
        this.placeData = placeData;
    }

    public PlaceNewItemEvent(String token) {
        this.token = token;
        this.placeClass = null;
        this.placeData = null;
    }

    @Override
    protected void dispatch(PlaceNewItemHandler handler) {
        handler.onPlaceNewItem(this);
    }

    @Override
    public Type<PlaceNewItemHandler> getAssociatedType() {
        return getType();
    }

    public Class<? extends Place<?>> getPlaceClass() {
        return placeClass;
    }

    public Object getPlaceData() {
        return placeData;
    }

    public String getToken() {
        return token;
    }
}
