/*
 * Copyright 2010 Gal Dolber.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.guit.junit;

import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.inject.Inject;
import com.google.inject.Injector;

import com.guit.client.command.CommandService;
import com.guit.client.command.action.Action;
import com.guit.client.command.action.ActionHandler;
import com.guit.client.command.action.CommandException;
import com.guit.client.command.action.Handler;
import com.guit.client.command.action.Response;
import com.guit.server.command.HandlerRouter;

public class CommandServiceMock implements CommandService {

    @Inject
    Injector injector;

    @Override
    @SuppressWarnings("unchecked")
    public <A extends Action<R>, R extends Response> void execute(A action, AsyncCallback<R> callback) {
        Class<? extends Action<R>> actionClass = (Class<? extends Action<R>>) action.getClass();
        if (actionClass.isAnnotationPresent(ActionHandler.class)) {
            Class<? extends Handler<A, R>> handlerClass = (Class<? extends Handler<A, R>>) findClass(actionClass.getAnnotation(ActionHandler.class).value());
            Handler<A, R> handler = injector.getInstance(handlerClass);
            handler = getRoutedHandler(action, handler);
            try {
                callback.onSuccess(handler.handle(action));
            } catch (CommandException caught) {
                callback.onFailure(caught);
            }
        } else {
            throw new IllegalStateException("The action " + actionClass.getCanonicalName() + " do not specify the @ActionHandler");
        }
    }

    @Override
    public <A extends Action<R>, R extends Response> void executeLater(A action, AsyncCallback<R> callback) {
        execute(action, callback);
    }

    @SuppressWarnings("unchecked")
    private <A extends Action<R>, R extends Response> Handler<A, R> getRoutedHandler(A action, Handler<A, R> rpcHandler) {
        if (rpcHandler instanceof HandlerRouter) {
            Class<? extends Handler<A, R>> routedClass = ((HandlerRouter<A, R>) rpcHandler).getRoutedHandler(action);
            if (routedClass != null) {
                rpcHandler = injector.getInstance(routedClass);
            } else {
                return rpcHandler;
            }
            return getRoutedHandler(action, rpcHandler);
        }
        return rpcHandler;
    }
    
    private Class<?> findClass(String className) {
        try {
            return Class.forName(className);
        } catch (ClassNotFoundException e) {
            throw new IllegalStateException("Class not found. Found: " + className);
        }
    }
}
