/*
 * Copyright 2010 Gal Dolber.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.guit.junit;

import com.google.gwt.i18n.client.Constants;
import com.google.inject.TypeLiteral;
import com.google.inject.spi.TypeEncounter;
import com.google.inject.spi.TypeListener;

import com.guit.client.binder.ViewField;
import com.guit.client.binder.ViewInitializer;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;

public class GuitTestListener implements TypeListener {

    private void collectInitializers(ArrayList<Method> declaredMethods, Class<?> clazz) {
        declaredMethods.addAll(Arrays.asList(clazz.getDeclaredMethods()));
        Class<?> superclass = clazz.getSuperclass();
        if (superclass != null && !superclass.equals(Object.class)) {
            collectInitializers(declaredMethods, superclass);
        }
    }

    @Override
    public <I> void hear(TypeLiteral<I> typeLiteral, TypeEncounter<I> typeEncounter) {
        for (Field field : ((Class<? super I>) typeLiteral.getRawType()).getDeclaredFields()) {
            if (field.isAnnotationPresent(ViewField.class)) {
                if (Constants.class.isAssignableFrom(field.getType())) {
                    typeEncounter.register(new ConstantInjector<I>(field));
                } else {
                    typeEncounter.register(new ViewFieldInjector<I>(field));
                }
            }
        }

        ArrayList<Method> declaredMethods = new ArrayList<Method>();
        collectInitializers(declaredMethods, typeLiteral.getRawType());
        for (Method method : declaredMethods) {
            if (method.isAnnotationPresent(ViewInitializer.class)) {
                typeEncounter.register(new PresenterInitializerInjector<I>(method));
            }
        }
    }
}
