/*
 * Copyright 2010 Gal Dolber.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.guit.rebind.binder.contributor;

import com.google.gwt.core.client.GWT;
import com.google.gwt.core.ext.GeneratorContext;
import com.google.gwt.core.ext.TreeLogger;
import com.google.gwt.core.ext.UnableToCompleteException;
import com.google.gwt.core.ext.typeinfo.JMethod;
import com.google.gwt.core.ext.typeinfo.JParameter;
import com.google.gwt.user.rebind.StringSourceWriter;

import com.guit.client.binder.ContributorAnnotation;
import com.guit.client.binder.contributor.Debug;
import com.guit.rebind.binder.BinderContext;
import com.guit.rebind.binder.BinderContributor;

@ContributorAnnotation(Debug.class)
public class DebugContributor implements BinderContributor {

    private static final String GWTCANONICALNAME = GWT.class.getCanonicalName();

    @Override
    public void contribute(BinderContext binderContext, TreeLogger logger, GeneratorContext context) throws UnableToCompleteException {
        JMethod m = binderContext.getMethod();
        JParameter[] parameters = m.getParameters();

        StringSourceWriter writer = new StringSourceWriter();
        writer.println("if (!GWT.isScript()) {");
        String title = "@Debug " + binderContext.getPresenterType().getQualifiedSourceName() + "." + m.getName();
        log(writer, title);
        String[] parameterGetters = binderContext.getParameterGetters();
        for (int n = 0; n < parameters.length; n++) {
            JParameter p = parameters[n];
            log(writer, "@Debug----> " + p.getName() + ": " + "\" + " + parameterGetters[n] + " + \"");
        }
        writer.println("}");
        binderContext.addBeforeHandler(writer);

    }

    private void log(StringSourceWriter writer, String log) {
        writer.println(GWTCANONICALNAME + ".log(\"" + log + "\");");
    }

    @Override
    public String name() {
        return "Debug";
    }
}
